@@ -3,7 +3,7 @@
  *  Implementation of relations between expressions */
 
 /*
- *  GiNaC Copyright (C) 1999-2004 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2005 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include <iostream>
@@ -338,13 +338,13 @@ relational::operator relational::safe_bool() const
 		case not_equal:
 			return make_safe_bool(!ex_to<numeric>(df).is_zero());
 		case less:
-			return make_safe_bool(ex_to<numeric>(df)<_num0);
+			return make_safe_bool(ex_to<numeric>(df)<(*_num0_p));
 		case less_or_equal:
-			return make_safe_bool(ex_to<numeric>(df)<=_num0);
+			return make_safe_bool(ex_to<numeric>(df)<=(*_num0_p));
 		case greater:
-			return make_safe_bool(ex_to<numeric>(df)>_num0);
+			return make_safe_bool(ex_to<numeric>(df)>(*_num0_p));
 		case greater_or_equal:
-			return make_safe_bool(ex_to<numeric>(df)>=_num0);
+			return make_safe_bool(ex_to<numeric>(df)>=(*_num0_p));
 		default:
 			throw(std::logic_error("invalid relational operator"));
 	}
